@@ -436,9 +436,22 @@ iiop_connection_from_fd(int fd, IIOPConnection *parent)
   switch(parent->icnxtype) {
   case IIOP_IPV4:
     n = sizeof(struct sockaddr_in);
-    getpeername(GIOP_CONNECTION_GET_FD(fd_cnx), (struct sockaddr *)&fd_cnx->u.ipv4.location, &n);
-    hent = gethostbyaddr((const char *)&fd_cnx->u.ipv4.location.sin_addr.s_addr, 4, AF_INET);
-    fd_cnx->u.ipv4.hostname = g_strdup(hent->h_name);
+    if(getpeername(GIOP_CONNECTION_GET_FD(fd_cnx), (struct sockaddr *)&fd_cnx->u.ipv4.location, &n))
+      {
+	fd_cnx->u.ipv4.hostname = g_strdup("");
+      }
+    else
+      {
+	hent = gethostbyaddr((const char *)&fd_cnx->u.ipv4.location.sin_addr.s_addr, 4, AF_INET);
+	if(hent)
+	  {
+	    fd_cnx->u.ipv4.hostname = g_strdup(hent->h_name);
+	  }
+	else
+	  {
+	    fd_cnx->u.ipv4.hostname = inet_ntoa(*((struct in_addr *)&fd_cnx->u.ipv4.location.sin_addr));
+	  }
+      }
     break;
 
   case IIOP_USOCK:
@@ -490,7 +503,7 @@ iiop_connection_from_fd(int fd, IIOPConnection *parent)
 IIOPConnection *
 iiop_connection_server(void)
 {
-  struct hostent *hent, *hent2;
+  struct hostent *hent;
   char hn_tmp[65];
   socklen_t n;
   IIOPConnection *server_cnx = g_new0(IIOPConnection, 1);
@@ -525,13 +538,18 @@ iiop_connection_server(void)
   gethostname(hn_tmp, sizeof(hn_tmp) - 1);
 
   hent = gethostbyname(hn_tmp);
-  if(hent) {
-    hent2 = gethostbyaddr(hent->h_addr, 4, AF_INET);
-    if(hent2)
-      server_cnx->u.ipv4.hostname = g_strdup(hent2->h_name);
-    else
-      server_cnx->u.ipv4.hostname = g_strdup(inet_ntoa(*((struct in_addr *)hent->h_addr)));
-  } else
+  if(hent) 
+    {
+      if (strchr (hent->h_name, '.'))
+	server_cnx->u.ipv4.hostname = g_strdup(hent->h_name);
+      else
+	{
+	  struct in_addr * addr = (struct in_addr *) hent->h_addr_list[0];
+	  g_assert (hent->h_length == sizeof (struct in_addr) && addr);
+	  server_cnx->u.ipv4.hostname = g_strdup (inet_ntoa (*addr));
+	}
+    }
+  else
     server_cnx->u.ipv4.hostname = g_strdup(hn_tmp);
 
   listen(GIOP_CONNECTION_GET_FD(server_cnx), 5);
@@ -726,8 +744,11 @@ iiop_unlink_unix_sockets(void)
 
   for (item = iiop_unix_socket_list;
        item; item = g_slist_next(item)) {
-    if(GIOP_CONNECTION(item->data)->connection_class == GIOP_CONNECTION_SERVER)
-      unlink(IIOP_CONNECTION(item->data)->u.usock.sun_path);
+    GIOPConnection *cnx;
+
+    cnx = GIOP_CONNECTION(item->data);
+    if(cnx->connection_class == GIOP_CONNECTION_SERVER)
+      unlink(IIOP_CONNECTION(cnx)->u.usock.sun_path);
   }
 
   if (iiop_unix_socket_list) {
@@ -1045,6 +1066,8 @@ iiop_connection_server_accept(GIOPConnection *connection)
     GIOP_CONNECTION(newcnx)->orb_data = connection->orb_data;
     switch(IIOP_CONNECTION(connection)->icnxtype) {
     case IIOP_USOCK: newcnx->is_auth = TRUE; break;
+    default:
+      break;
     }
   }
 }
