@@ -171,7 +171,7 @@ gint
 giop_send_buffer_write(GIOPSendBuffer *send_buffer)
 {
   gulong nvecs;
-  glong res, i, sum, t;
+  glong res, sum, t;
   struct iovec *curvec;
   int fd;
   GIOPConnection *cnx;
@@ -214,22 +214,22 @@ giop_send_buffer_write(GIOPSendBuffer *send_buffer)
        5 + 2 !> 7
      */
 
-    for(t = 0; ; curvec++, nvecs--, t += curvec->iov_len) {
+    for(t = 0; ; t += curvec->iov_len, curvec++, nvecs--) {
       if((t + curvec->iov_len) > res)
 	break;
     }
     if((res - t) > 0) {
       curvec->iov_len -= (res - t);
-      curvec->iov_base += (res - t);
+      curvec->iov_base = (gpointer)((char *)curvec->iov_base + (res - t));
     }
 
     fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) & ~O_NONBLOCK);
 
-    res = writev(fd, curvec, nvecs);
+    t = writev(fd, curvec, nvecs);
 
     fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
 
-    if(res < sum) {
+    if((t < 0) || ((res + t) < sum)) {
 	giop_main_handle_connection_exception(cnx);
 	goto out;
     }
