@@ -49,7 +49,7 @@ struct CORBA_Request_type {
 };
 
 static const ORBit_RootObject_Interface interface_CORBA_Request = {
-	(gpointer)CORBA_Request_delete
+	(void (*)(gpointer,CORBA_Environment *))CORBA_Request_delete
 };
 
 /* Section 5.2.1 */
@@ -437,43 +437,3 @@ CORBA_NVList_get_count(CORBA_NVList *list,
 	*count = list->list->len;
 }
 
-/********* XXX todo - CORBA_Context support */
-/* Section 5.6.2 */
-CORBA_Status CORBA_Context_set_one_value(CORBA_Context ctx, CORBA_Identifier prop_name, char *value, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-}
-
-/* Section 5.6.3 */
-CORBA_Status CORBA_Context_set_values(CORBA_Context ctx, CORBA_NVList values, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	return;
-}
-
-/* Section 5.6.4 */
-CORBA_Status CORBA_Context_get_values(CORBA_Context ctx, CORBA_Identifier start_scope, CORBA_Flags op_flags, CORBA_Identifier prop_name, CORBA_NVList **values, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	
-}
-
-/* Section 5.6.5 */
-CORBA_Status CORBA_Context_delete_values(CORBA_Context ctx, CORBA_Identifier prop_name, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	
-}
-
-/* Section 5.6.6 */
-CORBA_Status CORBA_Context_create_child(CORBA_Context ctx, CORBA_Identifier ctx_name, CORBA_Context *child_ctx, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-	
-}
-
-/* Section 5.6.7 */
-CORBA_Status CORBA_Context_delete(CORBA_Context ctx, CORBA_Flags del_flags, CORBA_Environment *ev)
-{
-	g_assert(!"Not yet implemented");
-}
