@@ -292,7 +292,8 @@ ORBit_DynAny_items_from_any(CORBA_TypeCode type,
 				ORBit_DynAny_items_from_any(type->discriminator,
 							    value, ev);
 
-			data_tc = ORBit_get_union_tag(type, (gpointer *)value);
+			/* XXX: I dont know if {value} should be updated? */
+			data_tc = ORBit_get_union_tag(type, (gpointer *)value, TRUE);
 
 			*value = ALIGN_ADDRESS(*value,
 					       ORBit_find_alignment(data_tc));
