@@ -97,7 +97,7 @@ void CORBA_exception_free(CORBA_Environment *ev)
 }
 
 void CORBA_exception_set(CORBA_Environment *ev, CORBA_exception_type major, 
-			 CORBA_char *except_repos_id, void *param)
+			 const CORBA_char *except_repos_id, void *param)
 {
 	g_assert(ev!=NULL);
 
@@ -177,6 +177,7 @@ CORBA_any *CORBA_exception_as_any(CORBA_Environment *ev)
 
 	ev->_any=g_new(CORBA_any, 1);
 	if(ev->_any!=NULL) {
+		/* XXX is this the correct type? */
 		ev->_any->_type = (CORBA_TypeCode)TC_CORBA_ExceptionDescription;
 		ev->_any->_value = ev->_params;
 		ev->_any->_release = 0;
