@@ -29,25 +29,6 @@
 
 #include "config.h"
 
-/* AIX requires this to be the first thing in the file.  */
-#ifndef __GNUC__
-# if HAVE_ALLOCA_H
-#  include <alloca.h>
-# else
-#  ifdef _AIX
- #pragma alloca
-#  else
-#   ifndef alloca /* predefined by HP cc +Olibcalls */
-char *alloca ();
-#   endif
-#  endif
-# endif
-#endif
-
-#if defined(__STRICT_ANSI__) && defined(__GNUC__)
-#define alloca __builtin_alloca
-#endif
-
 #include <unistd.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -63,6 +44,8 @@ char *alloca ();
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <utime.h>
+
 #include "../IIOP/iiop-endianP.h"
 #include "orbit.h"
 
@@ -235,6 +218,13 @@ ORBit_ORB_make_usock_connection(void)
 		}
 	}
 
+	{
+		struct utimbuf utb;
+		memset(&utb, 0, sizeof(utb));
+		utime(tmpstr->str, &utb);
+	}
+
+
 #ifdef WE_DONT_CARE_ABOUT_STUPID_2DOT0DOTX_KERNELS
 	g_string_sprintf(tmpstr, "/tmp/orbit-%s",
 			 g_get_user_name());
@@ -300,7 +290,7 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 	char *root_poa_ior=NULL, *root_poa_addr=NULL;
 	char *orb_id_opt=NULL;
 	char *ctmp;
-	CORBA_ORB orb=NULL;
+	static CORBA_ORB orb=NULL;
 
 	/* The variable addresses in these structs need to be assigned at
 	 * run-time if you want to compile with -pedantic
@@ -343,6 +333,9 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 	g_return_val_if_fail(ev != NULL, NULL);
 	o_return_val_if_fail(argc && argv && orb_identifier, NULL);
 
+	if(orb)
+		return orb;
+
 	pre_rc_options[0].arg = (void *) &nosysrc;
 	pre_rc_options[1].arg = (void *) &nouserrc;
 	options[0].arg = (void *) &no_iiop_server;
@@ -540,6 +533,7 @@ CORBA_ORB CORBA_ORB_init(int *argc, char **argv, CORBA_ORBid orb_identifier, COR
 error:
 	if(orb!=NULL) {
 		ORBit_ORB_release(orb, ev);
+		orb = NULL;
 	}
 	g_free(imr_ior);
 	g_free(imr_addr);
@@ -594,7 +588,7 @@ static void ORBit_emit_profile(gpointer item, gpointer userdata)
 		CDR_put_ushort(encaps, profile->tag.iopinfo.port);
 		CDR_put_ulong(encaps, profile->object_key._length);
 		CDR_put_octets(encaps, profile->object_key._buffer,
-			profile->object_key._length);
+			       profile->object_key._length);
 		CDR_put_ulong(codec, encaps->wptr);
 		CDR_put_octets(codec, encaps->buffer, encaps->wptr);
 		break;
@@ -689,9 +683,50 @@ CORBA_char *CORBA_ORB_object_to_string(CORBA_ORB orb,
   }
   *ctmp = '\0';
 
+  {
+	  /* Debug check */
+	  CORBA_Object obj;
+	  CORBA_Environment myev;
+
+	  CORBA_exception_init(&myev);
+
+	  obj = CORBA_ORB_string_to_object(orb, rc, &myev);
+
+	  if (CORBA_Object_is_nil(obj, &myev)) {
+		  g_warning("Bug in %s, created bad IOR `%s'\n",
+			    __FUNCTION__, rc);
+		  CORBA_free(rc);
+		  return NULL;
+	  }
+			  
+	  CORBA_Object_release(obj, &myev);
+  }
+  
   return rc;
 }
 
+/* Quote from the GNU libc manual:
+
+   "If you try to allocate more storage than the machine can provide,
+    you don't get a clean error message. Instead you get a fatal
+    signal like the one you would get from an infinite recursion;
+    probably a segmentation violation (see section Program Error
+    Signals)."
+
+   The man page claims alloca() returns NULL on failure; this appears
+   to be a load of shit on Linux where you just get flaming death, but
+   we check anyway in case other systems work that way.
+
+   On Linux we check that the size is less than MAX_STACK_ALLOC
+
+   Note that the CORBA_alloc() calls in here can still cause
+   program abort, and really that should be fixed in a similar
+   way since our lengths are coming in from unpredictable sources
+   like files or the network.
+*/
+
+#define MAX_STACK_ALLOC 8192
+
 CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 					CORBA_Environment *ev)
 {
@@ -748,79 +783,140 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 
 		object_info=g_new0(ORBit_Object_info, 1);
 
-		CDR_get_ulong(codec, &tag);
+		if (!CDR_get_ulong(codec, &tag))
+			goto error_out;
+
 		switch(tag) {
 		case IOP_TAG_INTERNET_IOP:
-			CDR_get_ulong(codec, &misclen);
+			if (!CDR_get_ulong(codec, &misclen))
+				goto error_out;
+			
 			CDR_codec_init_static(encaps_codec);
+
+			if (misclen > MAX_STACK_ALLOC)
+				goto error_out;
+			
 			encaps_codec->buffer = alloca(misclen);
+			if (encaps_codec->buffer == NULL)
+				/* misclen was probably junk */
+				goto error_out;
+			
 			encaps_codec->release_buffer = FALSE;
-			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
+			if(!CDR_buffer_gets(codec, encaps_codec->buffer, misclen))
+				goto error_out;
 
 			encaps_codec->buf_len = misclen;
 			encaps_codec->readonly = CORBA_TRUE;
-			CDR_get_octet(encaps_codec, &endian);
+			if(!CDR_get_octet(encaps_codec, &endian))
+				goto error_out;
 			encaps_codec->data_endian = endian;
 			encaps_codec->host_endian = FLAG_ENDIANNESS;
 
+			if (encaps_codec->data_endian > 1)
+				goto error_out;
+			
 			object_info->profile_type = IOP_TAG_INTERNET_IOP;
-			CDR_get_octet(encaps_codec, &object_info->iiop_major);
+			if(!CDR_get_octet(encaps_codec, &object_info->iiop_major))
+				goto error_out;
 			if(object_info->iiop_major != 1)
 				goto error_out;
-			CDR_get_octet(encaps_codec, &object_info->iiop_minor);
-			CDR_get_string(encaps_codec, &object_info->tag.iopinfo.host);
-			CDR_get_ushort(encaps_codec, &object_info->tag.iopinfo.port);
-			CDR_get_seq_begin(encaps_codec, &object_info->object_key._length);
+			if(!CDR_get_octet(encaps_codec, &object_info->iiop_minor))
+				goto error_out;
+			if(!CDR_get_string(encaps_codec, &object_info->tag.iopinfo.host))
+				goto error_out;
+			if(!CDR_get_ushort(encaps_codec, &object_info->tag.iopinfo.port))
+				goto error_out;
+			if(!CDR_get_seq_begin(encaps_codec, &object_info->object_key._length))
+				goto error_out;
+
 			object_info->object_key._maximum = 0;
 
 			/* The POA gives out ORBit_alloc()d profiles, so we have to too */
 			object_info->object_key._buffer = ORBit_alloc(object_info->object_key._length, NULL, NULL);
-			CDR_buffer_gets(encaps_codec, object_info->object_key._buffer,
-					object_info->object_key._length);
+			if(!CDR_buffer_gets(encaps_codec, object_info->object_key._buffer,
+					    object_info->object_key._length))
+				goto error_out;
+
 			ORBit_set_object_key(object_info);
 			profiles=g_slist_append(profiles, object_info);
 			break;
 
 		case IOP_TAG_MULTIPLE_COMPONENTS:
 			/* Just skip any multiple_components data, for now */
-			CDR_get_ulong(codec, &misclen);
+			if(!CDR_get_ulong(codec, &misclen))
+				goto error_out;
+
 			CDR_codec_init_static(encaps_codec);
+
+			if (misclen > MAX_STACK_ALLOC)
+				goto error_out;
+			
 			encaps_codec->buf_len = misclen;
 			encaps_codec->buffer = alloca(misclen);
+			if (encaps_codec->buffer == NULL)
+				/* misclen was probably junk */
+				goto error_out;
+
 			encaps_codec->release_buffer = FALSE;
 			encaps_codec->readonly = CORBA_TRUE;
-			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
+			if(!CDR_buffer_gets(codec, encaps_codec->buffer, misclen))
+				goto error_out;
 			break;
 
 		case IOP_TAG_ORBIT_SPECIFIC:
-			CDR_get_ulong(codec, &misclen);
+			if(!CDR_get_ulong(codec, &misclen))
+				goto error_out;
+
 			CDR_codec_init_static(encaps_codec);
 
+			if (misclen > MAX_STACK_ALLOC)
+				goto error_out;
+			
 			encaps_codec->buffer = alloca(misclen);
+			if (encaps_codec->buffer == NULL)
+				/* misclen was probably junk */
+				goto error_out;
+			
 			encaps_codec->release_buffer = FALSE;
-			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
+			if(!CDR_buffer_gets(codec, encaps_codec->buffer, misclen))
+				goto error_out;
 
 			encaps_codec->buf_len = misclen;
 			encaps_codec->readonly = CORBA_TRUE;
 
-			CDR_get_octet(encaps_codec, &endian);
+			if(!CDR_get_octet(encaps_codec, &endian))
+				goto error_out;
+
 			encaps_codec->data_endian = endian;
 			encaps_codec->host_endian = FLAG_ENDIANNESS;
 
+			if (encaps_codec->data_endian > 1)
+				goto error_out;
+			
 			object_info->profile_type=IOP_TAG_ORBIT_SPECIFIC;
-			CDR_get_octet(encaps_codec, &object_info->iiop_major);
+			if(!CDR_get_octet(encaps_codec, &object_info->iiop_major))
+				goto error_out;
+
 			if(object_info->iiop_major != 1)
 				goto error_out;
-			CDR_get_octet(encaps_codec, &object_info->iiop_minor);
-			CDR_get_string(encaps_codec, &object_info->tag.orbitinfo.unix_sock_path);
-			CDR_get_ushort(encaps_codec, &object_info->tag.orbitinfo.ipv6_port);
-			CDR_get_seq_begin(encaps_codec, &object_info->object_key._length);
+			if(!CDR_get_octet(encaps_codec, &object_info->iiop_minor))
+				goto error_out;
+
+			if(!CDR_get_string(encaps_codec, &object_info->tag.orbitinfo.unix_sock_path))
+				goto error_out;
+
+			if(!CDR_get_ushort(encaps_codec, &object_info->tag.orbitinfo.ipv6_port))
+				goto error_out;
+			if(!CDR_get_seq_begin(encaps_codec, &object_info->object_key._length))
+				goto error_out;
 			object_info->object_key._maximum = 0;
 
 			/* The POA gives out ORBit_alloc()d profiles, so we have to too */
 			object_info->object_key._buffer = ORBit_alloc(object_info->object_key._length, NULL, NULL);
-			CDR_buffer_gets(encaps_codec, object_info->object_key._buffer,
-					object_info->object_key._length);
+			if(!CDR_buffer_gets(encaps_codec, object_info->object_key._buffer,
+					    object_info->object_key._length))
+				goto error_out;
+
 			ORBit_set_object_key(object_info);
 			profiles=g_slist_append(profiles, object_info);
 			break;
@@ -828,39 +924,43 @@ CORBA_Object CORBA_ORB_string_to_object(CORBA_ORB orb, CORBA_char *str,
 			g_warning("Unknown tag 0x%x", tag);
 
 			/* Skip it */
-			CDR_get_ulong(codec, &misclen);
+			if(!CDR_get_ulong(codec, &misclen))
+				goto error_out;
+
 			CDR_codec_init_static(encaps_codec);
+
+			if (misclen > MAX_STACK_ALLOC)
+				goto error_out;
+			
 			encaps_codec->buf_len = misclen;
 			encaps_codec->buffer = alloca(misclen);
+			if (encaps_codec->buffer == NULL)
+				/* misclen was probably junk */
+				goto error_out;
+
 			encaps_codec->release_buffer = FALSE;
 			encaps_codec->readonly = CORBA_TRUE;
-			CDR_buffer_gets(codec, encaps_codec->buffer, misclen);
+			if(!CDR_buffer_gets(codec, encaps_codec->buffer, misclen))
+				goto error_out;
+
 			break;
 		}
 	}
 
-	retval = ORBit_create_object_with_info(profiles, type_id, orb, ev);
+	return ORBit_create_object_with_info(profiles, type_id, orb, ev);
 
  error_out:
 
+	if(object_info) {
+		CORBA_free(object_info->object_key._buffer);
+		g_free(object_info);
+		ORBit_delete_profiles(profiles);
+	}
+
+
 	return retval;
 }
 
-/* Section 5.6.1 */
-CORBA_Status CORBA_ORB_get_default_context(CORBA_ORB orb, CORBA_Context *ctx, CORBA_Environment *ev)
-{
-	g_return_if_fail(ev != NULL);
-	o_return_if_fail(orb && ctx);
-
-	/* I have no clue what this is supposed to be...
-	   v2.2 spec page 5-13 says a CORBA_Context is a list of
-	   name-value pairs, the orb.idl file says it's a string,
-	   and I have no idea which is correct
-	*/
-
-	*ctx = g_strdup("orbit-local-orb");
-}
-
 /* Section 4.1.2 */
 CORBA_boolean CORBA_ORB_get_service_information(CORBA_ORB orb, CORBA_ServiceType service_type, CORBA_ServiceInformation *service_information, CORBA_Environment *ev)
 {
