@@ -182,7 +182,7 @@ GIOP_RECV_BUFFER(recv_buffer)->cur = ((guchar *)GIOP_RECV_BUFFER(recv_buffer)->c
 
 CORBA_Object
 ORBit_create_object_with_info(GSList *profiles,
-			      CORBA_char *type_id,
+			      const CORBA_char *type_id,
 			      CORBA_ORB orb,
 			      CORBA_Environment *ev)
 {
