@@ -33,6 +33,10 @@
 #include "orbit.h"
 #include "orbit_poa_type.h"
 #include "orbit_poa.h"
+#include "genrand.h"
+
+#define POA_KEY_LEN (sizeof(CORBA_unsigned_long) + ORBIT_RAND_KEY_LEN)
+#define OBJ_KEY_LEN (sizeof(CORBA_unsigned_long) + ORBIT_RAND_KEY_LEN)
 
 static void ORBit_POAManager_release(PortableServer_POAManager poa_mgr,
 				     CORBA_Environment *ev);
@@ -44,15 +48,17 @@ static PortableServer_Servant
 ORBit_POA_ServantManager_use_servant(PortableServer_POA poa,
 				     GIOPRecvBuffer *recv_buffer,
 				     PortableServer_ServantLocator_Cookie *the_cookie,
+				     PortableServer_ObjectId *oid,
+				     ORBit_POAObject *fake_obj_impl,
 				     CORBA_Environment *ev);
 static void
 ORBit_POA_ServantManager_unuse_servant(PortableServer_Servant servant,
 				       PortableServer_POA poa,
 				       GIOPRecvBuffer *recv_buffer,
 				       PortableServer_ServantLocator_Cookie cookie,
+				       PortableServer_ObjectId *oid,
+				       ORBit_POAObject *fake_obj_impl,
 				       CORBA_Environment *ev);
-					   
-
 
 static const ORBit_RootObject_Interface CORBA_POAManager_epv =
 {
@@ -303,6 +309,8 @@ ORBit_POA_new(CORBA_ORB orb,
 	g_ptr_array_set_size(poa->objnum_to_obj, 1);
 	g_ptr_array_index(poa->objnum_to_obj, 0) = NULL;
 
+	orbit_genrand(poa->rand_data, ORBIT_RAND_KEY_LEN);
+
 	return poa;
 
 error:
@@ -363,24 +371,23 @@ ORBit_POA_remove_child(PortableServer_POA poa,
 	poa->child_POAs = g_slist_remove(poa->child_POAs, child_poa);
 }
 
+extern ORBit_request_validate ORBIT_request_validator;
+
 gboolean
 ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 			 PortableServer_POA poa)
 {
 	PortableServer_ServantBase *servant;
 	PortableServer_ServantLocator_Cookie cookie;
-	ORBit_POAObject *obj_impl;
+	ORBit_POAObject *obj_impl = NULL, tmp_obj_impl;
 	ORBitSkeleton skel;
 	gpointer imp;
 	CORBA_Environment ev;
 	GIOPSendBuffer *send_buffer;
-
-	g_assert(poa);
-	g_assert(recv_buffer);
+	guchar *opname;
+	PortableServer_ObjectId *oid = NULL;
 
 	CORBA_exception_init(&ev);
-	
-	g_assert(poa);
 
 	switch(poa->the_POAManager->state) {
 	case PortableServer_POAManager_HOLDING:
@@ -425,25 +432,20 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 
 	servant = NULL;
 
-	if(poa->servant_retention == PortableServer_RETAIN) {
-		gpointer nptr;
-		CORBA_unsigned_long onum;
+	if(recv_buffer->message.u.request.object_key._length
+	   < (POA_KEY_LEN + sizeof(CORBA_unsigned_long))) {
+		CORBA_exception_set_system(&ev,
+					   ex_CORBA_OBJECT_NOT_EXIST,
+					   CORBA_COMPLETED_NO);
+		goto errout;
+	}
 
-		if(recv_buffer->message.u.request.object_key._length
-		   != sizeof(CORBA_unsigned_long) * 2) {
-			CORBA_exception_set_system(&ev,
-						   ex_CORBA_OBJECT_NOT_EXIST,
-						   CORBA_COMPLETED_NO);
-			goto errout;
-		}
+	obj_impl = ORBit_POA_find_oid_for_object_key(poa, &(recv_buffer->message.u.request.object_key), &oid);
 
-		nptr = recv_buffer->message.u.request.object_key._buffer;
-
-		onum = *((CORBA_unsigned_long *)((guchar *)nptr + sizeof(CORBA_unsigned_long)));
-		obj_impl = g_ptr_array_index(poa->objnum_to_obj, onum);
-		if(obj_impl
-		   && GPOINTER_TO_UINT(obj_impl) > poa->objnum_to_obj->len)
-			servant = obj_impl->servant;
+	if(poa->servant_retention == PortableServer_RETAIN
+	   && obj_impl) {
+		servant = obj_impl->servant;
+		oid = obj_impl->object_id;
 	}
 
 	if(!servant) {
@@ -452,51 +454,75 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 			servant = ORBit_POA_ServantManager_use_servant(poa,
 								       recv_buffer,
 								       &cookie,
+								       oid,
+								       &tmp_obj_impl,
 								       &ev);
 			break;
 		case PortableServer_USE_DEFAULT_SERVANT:
 			servant = poa->default_servant;
-			if(servant == NULL)
+			if(servant == NULL) {
 				CORBA_exception_set_system(&ev,
 							   ex_CORBA_OBJ_ADAPTER,
 							   CORBA_COMPLETED_NO);
+				goto errout;
+			}
 			break;
-		case PortableServer_USE_ACTIVE_OBJECT_MAP_ONLY:
 		default:
-			CORBA_exception_set_system(&ev,
-						   ex_CORBA_OBJECT_NOT_EXIST,
-						   CORBA_COMPLETED_NO);
-			goto errout;
+			break;
 		}
 	}
 
-	g_assert(poa);
-	g_assert(servant);
-	g_assert(servant->_private);
+	if(!poa || !servant || !servant->_private) {
+		CORBA_exception_set_system(&ev,
+					   ex_CORBA_OBJECT_NOT_EXIST,
+					   CORBA_COMPLETED_NO);
+		goto errout;
+	}
+
+	opname = recv_buffer->message.u.request.operation;
 
 	skel = ORBIT_OBJECT_KEY(servant->_private)->class_info->relay_call(servant, recv_buffer, &imp);
-	if(!skel && recv_buffer->message.u.request.operation[0] == '_') {
+	
+	if(!skel && opname[0] == '_') {
 		guchar *opname = recv_buffer->message.u.request.operation;
 
 		if(!strcmp(opname, "_is_a"))
 			skel = (gpointer)&ORBit_impl_CORBA_Object_is_a;
 	}
 
-	if(!skel) {
+	if (!skel) {
 		CORBA_exception_set_system(&ev, ex_CORBA_BAD_OPERATION,
 					   CORBA_COMPLETED_NO);
 		goto errout;
 	}
 
+	if (!imp) {
+		CORBA_exception_set_system(&ev, ex_CORBA_NO_IMPLEMENT,
+					   CORBA_COMPLETED_NO);
+		goto errout;
+	}
+
+	/* If it got through the random keys, and nobody else had the opportunity to say otherwise, it must be auth'd */
+
+	if(!ORBIT_request_validator)
+		GIOP_MESSAGE_BUFFER(recv_buffer)->connection->is_auth = TRUE;
+
 	skel(servant, recv_buffer, &ev, imp);
 
 	if(poa->request_processing == PortableServer_USE_SERVANT_MANAGER) {
 		ORBit_POA_ServantManager_unuse_servant(servant,
 						       poa,
 						       recv_buffer,
-						       cookie, &ev);
+						       cookie, oid,
+						       &tmp_obj_impl,
+						       &ev);
 	}
 
+	if(!obj_impl)
+		CORBA_free(oid);
+
+	CORBA_exception_free(&ev);
+
 	return TRUE;
 
  errout:
@@ -516,6 +542,9 @@ ORBit_POA_handle_request(GIOPRecvBuffer *recv_buffer,
 	} else /* User exceptions are handled in the skels! */
 		g_assert(ev._major == CORBA_NO_EXCEPTION);
 
+	if(!obj_impl)
+		CORBA_free(oid);
+
 	CORBA_exception_free(&ev);
 
 	return TRUE;
@@ -527,25 +556,104 @@ ORBit_POA_find_POA_for_object_key(PortableServer_POA root_poa,
 {
 	CORBA_unsigned_long pid;
 
+	if(key->_length < (sizeof(CORBA_unsigned_long) + ORBIT_RAND_KEY_LEN))
+		return NULL;
+
 	pid = *((CORBA_unsigned_long *)key->_buffer);
-	if(pid < root_poa->orb->poas->len)
-		return g_ptr_array_index(root_poa->orb->poas, pid);
-	else
+
+	if(pid < root_poa->orb->poas->len) {
+		PortableServer_POA poa;
+		poa = g_ptr_array_index(root_poa->orb->poas, pid);
+		if(!poa)
+			return NULL;
+		if(memcmp(poa->rand_data, key->_buffer + sizeof(CORBA_unsigned_long), ORBIT_RAND_KEY_LEN))
+			return NULL;
+		return poa;
+	} else
 		return NULL;
 }
 
 void
 ORBit_POA_find_object_key_for_oid(PortableServer_POA poa,
 				  ORBit_POAObject *obj,
+				  PortableServer_ObjectId *oid,
 				  CORBA_sequence_octet *retval)
 {
-	g_return_if_fail(poa && obj);
+	CORBA_long *vptr;
 
-	retval->_length = sizeof(CORBA_unsigned_long) * 2;
+	g_return_if_fail(poa && (obj || oid));
+	g_return_if_fail(retval);
+
+	if(oid)
+		g_assert(!oid->_buffer[oid->_length - 1]);
+
+	if(obj)
+		retval->_length = POA_KEY_LEN + OBJ_KEY_LEN;
+	else
+		retval->_length = POA_KEY_LEN  + sizeof(CORBA_long) + oid->_length;
 	retval->_buffer = CORBA_octet_allocbuf(retval->_length);
 	CORBA_sequence_set_release(retval, CORBA_TRUE);
-	*((CORBA_unsigned_long *)retval->_buffer) = poa->poaID;
-	*((CORBA_unsigned_long *)(retval->_buffer + sizeof(CORBA_unsigned_long))) = obj->objnum;
+
+	vptr = (CORBA_long *)retval->_buffer;
+	*vptr = poa->poaID;
+	memcpy(retval->_buffer + sizeof(CORBA_unsigned_long), poa->rand_data, ORBIT_RAND_KEY_LEN);
+
+	vptr = (CORBA_long *)(retval->_buffer + POA_KEY_LEN);
+	if(obj) {
+		*vptr = obj->objnum;
+		memcpy(retval->_buffer + POA_KEY_LEN + sizeof(CORBA_unsigned_long), obj->rand_data, ORBIT_RAND_KEY_LEN);
+	} else {
+		*vptr = -((CORBA_long)oid->_length);
+		memcpy(retval->_buffer + POA_KEY_LEN + sizeof(CORBA_unsigned_long), oid->_buffer, oid->_length);
+	}
+}
+
+ORBit_POAObject *
+ORBit_POA_find_oid_for_object_key(PortableServer_POA poa,
+				  CORBA_sequence_octet *object_key,
+				  PortableServer_ObjectId **oid)
+{
+	CORBA_long onum;
+	guchar *nptr;
+	ORBit_POAObject *objinfo;
+
+	*oid = NULL;
+	nptr = object_key->_buffer + POA_KEY_LEN;
+
+	if(object_key->_length < (POA_KEY_LEN + sizeof(CORBA_long))) {
+		return NULL;
+	}
+
+	onum = *((CORBA_long *)nptr);
+
+	if(onum < 0) {
+		/* onum will be the -strlen(ObjectId) */
+		if(object_key->_length < (POA_KEY_LEN + sizeof(CORBA_long) - onum))
+			return NULL;
+
+		*oid = (PortableServer_ObjectId *)CORBA_sequence_octet__alloc();
+		(*oid)->_length = -onum;
+		(*oid)->_buffer = CORBA_octet_allocbuf((*oid)->_length);
+		memcpy((*oid)->_buffer, object_key->_buffer + POA_KEY_LEN + sizeof(CORBA_long), (*oid)->_length);
+
+		return NULL;
+	}
+
+	if(onum >= poa->objnum_to_obj->len)
+		return NULL;
+
+	objinfo = g_ptr_array_index(poa->objnum_to_obj, onum);
+
+	if(GPOINTER_TO_UINT(objinfo) <= poa->objnum_to_obj->len)
+		return NULL;
+
+	if(object_key->_length < (POA_KEY_LEN + OBJ_KEY_LEN))
+		return NULL;
+
+	if(memcmp(object_key->_buffer + POA_KEY_LEN + sizeof(CORBA_long), objinfo->rand_data, ORBIT_RAND_KEY_LEN))
+		return NULL;
+
+	return objinfo;
 }
 
 DEFINE_LOCK(id_assignment_counter);
@@ -570,6 +678,7 @@ ORBit_POA_allocate_oid(PortableServer_POA poa,
 	len = strlen(buf)+1;
 	new_objid->_buffer = CORBA_octet_allocbuf(len);
 	new_objid->_length = len;
+	new_objid->_maximum = len;
 	new_objid->_release = CORBA_TRUE;
 
 	strcpy((CORBA_char *)new_objid->_buffer, buf);
@@ -581,31 +690,40 @@ static PortableServer_Servant
 ORBit_POA_ServantManager_use_servant(PortableServer_POA poa,
 				     GIOPRecvBuffer *recv_buffer,
 				     PortableServer_ServantLocator_Cookie *the_cookie,
+				     PortableServer_ObjectId *oid,
+				     ORBit_POAObject *fake_obj_impl,
 				     CORBA_Environment *ev)
 {
-	PortableServer_ObjectId oid;
-
-	memcpy(&oid, &recv_buffer->message.u.request.object_key, sizeof(oid));
-	oid._buffer += sizeof(CORBA_unsigned_long);
-	oid._length -= sizeof(CORBA_unsigned_long);
-
 	if(poa->servant_retention == PortableServer_RETAIN) {
 		POA_PortableServer_ServantActivator *sm;
 		POA_PortableServer_ServantActivator__epv *epv;
 		
 		sm = (POA_PortableServer_ServantActivator *)poa->servant_manager;
 		epv = sm->vepv->PortableServer_ServantActivator_epv;
-		return epv->incarnate(sm, &oid, poa, ev);
+		return epv->incarnate(sm, oid, poa, ev);
 	} else {
 		POA_PortableServer_ServantLocator *sm;
 		POA_PortableServer_ServantLocator__epv *epv;
+		PortableServer_ServantBase *retval;
 
 		sm = (POA_PortableServer_ServantLocator *)poa->servant_manager;
 		epv = sm->vepv->PortableServer_ServantLocator_epv;
-		return epv->preinvoke(sm, &oid,
-				      poa, recv_buffer->message.u.request.operation,
-				      the_cookie,
-				      ev);
+		retval = epv->preinvoke(sm, oid,
+					poa, recv_buffer->message.u.request.operation,
+					the_cookie,
+					ev);
+
+		((ORBit_ObjectKey *)retval->_private)->object = fake_obj_impl;
+		fake_obj_impl->object_id = oid;
+		fake_obj_impl->poa = poa;
+		fake_obj_impl->orb = poa->orb;
+		fake_obj_impl->objnum = -1;
+#ifdef NOT_BACKWARDS_COMPAT_0_4
+		fake_obj_impl->use_count = NULL;
+		fake_obj_impl->death_callback = NULL;
+#endif
+
+		return retval;
 	}
 }
 
@@ -614,26 +732,24 @@ ORBit_POA_ServantManager_unuse_servant(PortableServer_Servant servant,
 				       PortableServer_POA poa,
 				       GIOPRecvBuffer *recv_buffer,
 				       PortableServer_ServantLocator_Cookie cookie,
+				       PortableServer_ObjectId *oid,
+				       ORBit_POAObject *fake_obj_impl,
 				       CORBA_Environment *ev)
 {
-	PortableServer_ObjectId oid;
 	POA_PortableServer_ServantLocator *sm;
 	POA_PortableServer_ServantLocator__epv *epv;
 
 	if(poa->servant_retention != PortableServer_NON_RETAIN)
 		return;
 
-	memcpy(&oid, &recv_buffer->message.u.request.object_key, sizeof(oid));
-	oid._buffer += sizeof(CORBA_unsigned_long);
-	oid._length -= sizeof(CORBA_unsigned_long);
-
-
 	sm = (POA_PortableServer_ServantLocator *)poa->servant_manager;
 	epv = sm->vepv->PortableServer_ServantLocator_epv;
 	
-	epv->postinvoke(sm, &oid,
+	((ORBit_ObjectKey *)((PortableServer_ServantBase *)servant)->_private)->object = NULL;
+	epv->postinvoke(sm, oid,
 			poa, recv_buffer->message.u.request.operation,
 			cookie, servant, ev);
+
 }
 
 typedef struct {
@@ -678,3 +794,19 @@ ORBit_POA_etherealize_objects(PortableServer_POA poa,
 				     &ei);
 	}
 }
+
+#ifdef NOT_BACKWARDS_COMPAT_0_4
+void ORBit_servant_set_deathwatch(PortableServer_ServantBase *servant,
+				  int *use_count,
+                                  GFunc death_func,
+                                  gpointer user_data)
+{
+	ORBit_POAObject *pobj;
+
+	pobj = ORBIT_OBJECT_KEY(servant->_private)->object;
+
+	pobj->use_count = use_count;
+	pobj->death_callback = death_func;
+	pobj->user_data = user_data;
+}
+#endif
