@@ -20,7 +20,7 @@
 
 gboolean orb_server_keep_running = FALSE;
 
-static ORBit_request_validate request_validator = NULL;
+ORBit_request_validate ORBIT_request_validator = NULL;
 
 /* function protos */
 static PortableServer_POA ORBit_find_POA_for_request(PortableServer_POA poa,
@@ -70,10 +70,10 @@ ORBit_handle_incoming_request(GIOPRecvBuffer *recv_buffer)
 		    recv_buffer->message.u.request.request_id,
 		    recv_buffer->message.u.request.object_key._buffer);
 
-	if(request_validator)
-		mvr = request_validator(recv_buffer->message.u.request.request_id,
-				&recv_buffer->message.u.request.requesting_principal,
-				recv_buffer->message.u.request.operation);
+	if(ORBIT_request_validator)
+		mvr = ORBIT_request_validator(recv_buffer->message.u.request.request_id,
+					      &recv_buffer->message.u.request.requesting_principal,
+					      recv_buffer->message.u.request.operation);
 	else
 		mvr = ORBIT_MESSAGE_ALLOW;
 
@@ -194,5 +194,5 @@ ORBit_find_POA_for_locate_request(PortableServer_POA poa,
 void
 ORBit_set_request_validation_handler(ORBit_request_validate validator)
 {
-	request_validator = validator;
+	ORBIT_request_validator = validator;
 }
